@@ -1,4 +1,5 @@
 #![allow(clippy::len_zero, clippy::redundant_closure, clippy::bool_comparison)]
+
 use crate::arg::ArgAttrData;
 use crate::macro_attribute::{MacroAttribute, NameValueAttribute};
 use crate::option::OptionAttrData;
@@ -51,11 +52,8 @@ impl CommandAttrData {
     pub fn from_fn(args: AttributeArgs, func: ItemFn) -> Self {
         let name = func.sig.ident.to_string();
         let attr_data =
-            NameValueAttribute::from_attribute_args(
-                name.as_str(),
-                args,
-                AttrStyle::Outer
-        ).expect("failed to parse `command` attribute");
+            NameValueAttribute::from_attribute_args(name.as_str(), args, AttrStyle::Outer)
+                .expect("failed to parse `command` attribute");
 
         CommandAttrData::new_from_fn(attr_data, func, false, true, true)
     }
@@ -154,6 +152,8 @@ impl CommandAttrData {
             )
         }
 
+        // Infer the global options
+
         self.children.push(command);
     }
 
@@ -197,7 +197,7 @@ impl CommandAttrData {
         if self.vars.contains(&var) {
             panic!(
                 "duplicated variable: `{}` in `{}`",
-                var.name(),
+                var.var_name(),
                 self.fn_name.name()
             )
         }
@@ -230,13 +230,21 @@ impl CommandAttrData {
         None
     }
 
-    pub fn expand(&self) -> TokenStream {
+    pub fn expand(mut self) -> TokenStream {
         assert!(
             self.item_fn.is_some(),
             "`ItemFn` is not set for command `{}`",
             self.fn_name
         );
 
+        self.assert_global_options();
+
+        // Apply only to root
+        if !self.is_child {
+            self.infer_global_options();
+            self.update_children_global_options();
+        }
+
         // Command args
         let args = self
             .args
@@ -248,6 +256,7 @@ impl CommandAttrData {
         let options = self
             .options
             .iter()
+            .filter(|x| !x.is_from_global())
             .map(|x| quote! { .option(#x)})
             .collect::<Vec<TokenStream>>();
 
@@ -396,6 +405,125 @@ impl CommandAttrData {
         }
     }
 
+    fn infer_global_options(&self) {
+        /*
+        FIXME: This is a hack to infer the options which requires to make a copy
+        of the options, and then iterate through the children,
+
+        The wanted solution is to make a parent <-> child relation with comment
+        which make easy to navigate through the tree to locale the parent which contains
+        the global options. This solution would require to make the Command a `Rc<RefCell<CommandAttrData>>`
+        to allow the children command use `Weak<RefCell<CommandAttrData>>` to access the parent.
+        */
+
+        let global_options = self
+            .options
+            .iter()
+            .filter(|o| o.is_global())
+            .cloned()
+            .collect::<Vec<_>>();
+
+        global_options.iter().for_each(|o| {
+            self.set_global_options(o);
+        });
+
+        for child in self.children.iter() {
+            child.infer_global_options();
+        }
+    }
+
+    fn set_global_options(&self, option: &OptionAttrData) {
+        assert!(option.is_global());
+
+        for opt in self.options.iter() {
+            if opt.from_global.get().is_none() && opt.arg_name == option.arg_name {
+                if !opt.is_global() {
+                    opt.set_from_global(true);
+                }
+            }
+        }
+
+        for child in self.children.iter() {
+            child.set_global_options(option);
+        }
+    }
+
+    fn assert_global_options(&self) {
+        if self.is_child {
+            return;
+        }
+
+        let mut global_options = Vec::from_iter(self.options.iter().filter(|o| o.is_global()));
+
+        for child in &self.children {
+            for option in &child.options {
+                if option.is_from_global() {
+                    if !global_options.iter().any(|o| o.name() == option.name()) {
+                        panic!(
+                            "There is no global option named `{}` in a parent command",
+                            option.name()
+                        );
+                    }
+                }
+
+                if option.is_global() {
+                    global_options.push(option);
+                }
+            }
+        }
+    }
+
+    fn update_children_global_options(&mut self) {
+        /*
+        FIXME: This is a hack to update the children global options.
+
+        The wanted solution is to make a parent <-> child relation with comment
+        which make easy to navigate through the tree to locale the parent which contains
+        the global options. This solution would require to make the Command a `Rc<RefCell<CommandAttrData>>`
+        to allow the children command use `Weak<RefCell<CommandAttrData>>` to access the parent.
+        */
+
+        fn update_children_options(opt: &OptionAttrData, parent: &mut CommandAttrData) {
+            for child in parent.children.iter_mut() {
+                if let Some(child_opt) = child
+                    .options
+                    .iter_mut()
+                    .find(|o| o.arg_name == opt.arg_name)
+                {
+                    if child_opt.is_from_global() {
+                        // Updated the children and vars to match the global option names
+                        child_opt.name = opt.name().to_string();
+
+                        let var = child
+                            .vars
+                            .iter_mut()
+                            .find(|v| v.var_name == opt.arg_name)
+                            .unwrap();
+
+                        var.name = Some(opt.name().to_string());
+                    }
+                } else {
+                    update_children_options(opt, child);
+                }
+            }
+        }
+
+        let global_options = self
+            .options
+            .iter()
+            .filter(|o| o.is_global())
+            .cloned()
+            .collect::<Vec<_>>();
+
+        for opt in global_options {
+            update_children_options(&opt, self);
+        }
+
+        for child in self.children.iter_mut() {
+            child.update_children_global_options();
+        }
+    }
+
     fn get_body(&self, vars: &[TokenStream]) -> TokenStream {
         let ret = &self.item_fn.as_ref().unwrap().sig.output;
         let error_handling = match ret {
@@ -407,7 +535,7 @@ impl CommandAttrData {
         if self.is_child {
             let fn_name = self.fn_name.to_string().parse::<TokenStream>().unwrap();
             let inputs = self.vars.iter().map(|var| {
-                let var_name = var.name().parse::<TokenStream>().unwrap();
+                let var_name = var.var_name().parse::<TokenStream>().unwrap();
                 let is_ref = match var.arg_type() {
                     ArgumentType::Slice(ty) => {
                         if ty.mutability {
@@ -487,7 +615,7 @@ impl CommandAttrData {
 
 impl ToTokens for CommandAttrData {
     fn to_tokens(&self, tokens: &mut TokenStream) {
-        tokens.append_all(self.expand().into_iter())
+        tokens.append_all(self.clone().expand().into_iter())
     }
 }
 
@@ -504,6 +632,8 @@ impl PartialEq for CommandAttrData {
 pub struct FnArgData {
     // Name of the function argument
     pub arg_name: String,
+    // Name specified with `name="value"`, if any
+    pub name: Option<String>,
     // The function argument `PatType` like: `x : i64`
     pub pat_type: PatType,
     // The macro attribute if any, this is solely used for debugging,
@@ -621,9 +751,12 @@ pub fn is_option_bool_flag(fn_arg: &FnArgData) -> bool {
     if let Some(attribute) = &fn_arg.name_value {
         // #[option(flag=false)]
         if let Some(flag_value) = attribute.get(crate::consts::FLAG) {
-           if !flag_value.to_bool_literal().expect("`flag` must be a bool literal") {
-               return false;
-           }
+            if !flag_value
+                .to_bool_literal()
+                .expect("`flag` must be a bool literal")
+            {
+                return false;
+            }
         }
 
         let min = attribute
@@ -668,10 +801,10 @@ mod imp {
     use crate::utils::{path_to_string, NamePath};
     use crate::var::{ArgLocalVar, VarSource};
     use crate::{consts, AttrQuery};
+    use quote::ToTokens;
     use std::path::{Path, PathBuf};
     use std::sync::atomic::{AtomicBool, Ordering};
     use syn::{AttrStyle, Attribute, AttributeArgs, File, FnArg, Item, ItemFn, PatType, Stmt};
-    use quote::ToTokens;
 
     // Constructs a new `CommandAttrData` from a `ItemFn`
     pub fn command_from_fn_with_name(
@@ -777,11 +910,16 @@ mod imp {
                 } else {
                     VarSource::Opts(fn_arg.arg_name.clone())
                 };
-                command.set_var(ArgLocalVar::new(fn_arg.pat_type.clone(), source));
+                command.set_var(ArgLocalVar::new(
+                    fn_arg.pat_type.clone(),
+                    source,
+                    fn_arg.name.clone(),
+                ));
             } else {
                 command.set_var(ArgLocalVar::new(
                     fn_arg.pat_type.clone(),
                     VarSource::Args(fn_arg.arg_name.clone()),
+                    fn_arg.name.clone(),
                 ));
             }
         }
@@ -895,7 +1033,11 @@ mod imp {
             // Finds the `command_help` if any in the body of the function
             if let Some(command_help) = find_inner_decorate_item_fn(&item_fn, consts::COMMAND_HELP)
             {
-                assert!(command.command_help.is_none(), "multiple #[{}] defined", consts::COMMAND_HELP);
+                assert!(
+                    command.command_help.is_none(),
+                    "multiple #[{}] defined",
+                    consts::COMMAND_HELP
+                );
                 command.set_command_help(NamePath::new(command_help.sig.ident.to_string()));
             }
 
@@ -903,7 +1045,11 @@ mod imp {
             if let Some(command_usage) =
                 find_inner_decorate_item_fn(&item_fn, consts::COMMAND_USAGE)
             {
-                assert!(command.command_usage.is_none(), "multiple #[{}] defined", consts::COMMAND_USAGE);
+                assert!(
+                    command.command_usage.is_none(),
+                    "multiple #[{}] defined",
+                    consts::COMMAND_USAGE
+                );
                 command.set_command_usage(NamePath::new(command_usage.sig.ident.to_string()));
             }
         }
@@ -999,7 +1145,7 @@ mod imp {
             let (name, attribute, _) = &attributes[index];
             if attributes[(index + 1)..]
                 .iter()
-                .any(|(arg, ..)| arg == name)
+                .any(|(arg_name, ..)| arg_name == name)
             {
                 panic!(
                     "function argument `{}` is already used in `{}`",
@@ -1019,26 +1165,31 @@ mod imp {
         }
 
         for (arg_name, pat_type) in fn_args {
-            let (attribute, name_value) = if let Some(data) =
-                attributes.iter().find_map(|(path, attribute, name_value)| {
-                    if path == &arg_name {
-                        Some((attribute.clone(), name_value.clone()))
-                    } else {
-                        None
-                    }
-                }) {
-                (Some(data.0), Some(data.1))
+            let (attribute, name_value) = if let Some((macro_attr, name_value_attr)) =
+                find_attribute_by_arg_name(&attributes, &arg_name)
+            {
+                (Some(macro_attr), Some(name_value_attr))
             } else {
                 (None, None)
             };
 
+            // Whether if this argument should be considered an option
             let is_option = attribute
                 .as_ref()
                 .map(|attribute| crate::consts::is_option(attribute.path()))
                 .unwrap_or(true);
 
+            // Get the name defined by `name="value"`
+            let name = name_value
+                .as_ref()
+                .map(|name_value| name_value.get(consts::NAME))
+                .flatten()
+                .map(|x| x.to_string_literal())
+                .flatten();
+
             ret.push(FnArgData {
                 arg_name,
+                name,
                 pat_type,
                 attribute,
                 name_value,
@@ -1049,6 +1200,20 @@ mod imp {
         ret
     }
 
+    // Returns the attribute that match the argument name
+    fn find_attribute_by_arg_name(
+        attributes: &[(String, MacroAttribute, NameValueAttribute)],
+        arg_name: &str,
+    ) -> Option<(MacroAttribute, NameValueAttribute)> {
+        attributes.iter().find_map(|(path, attribute, name_value)| {
+            if path == arg_name {
+                Some((attribute.clone(), name_value.clone()))
+            } else {
+                None
+            }
+        })
+    }
+
     // Takes a `MacroAttribute` and returns its path, self and this name values
     fn split_attr_path_and_name_values(
         attribute: MacroAttribute,
